#include <ConsumerImplHelper/BlazeCamera.h>
#include <opencv2/opencv.hpp>

using namespace GenTLConsumerImplHelper;
using namespace std;
using namespace cv;


// Calculates a grayscale range map from point cloud data sent from a Basler ToF camera.
// The buffer that pRangeMap points to must be allocated accordingly before 
// passing it to the calculateRangeMap function.
void calculateRangeMap(const PartInfo& pointCloud, int minDepth, int maxDepth, uint16_t* pRangeMap)
{
    const int width = (int)pointCloud.width;
    const int height = (int)pointCloud.height;
    CBlazeCamera::Coord3D *pPoint = reinterpret_cast<CBlazeCamera::Coord3D*>(pointCloud.pData);

    const double scale = 65535.0 / (maxDepth - minDepth);

    for (int row = 0; row < height; ++row)
    {
        for (int col = 0; col < width; ++col, ++pPoint, ++pRangeMap)
        {
            if (pPoint->IsValid())
            {
                // calculate the radial distance
                double distance = sqrt(pPoint->x * pPoint->x + pPoint->y * pPoint->y + pPoint->z * pPoint->z);
                // clip to [minDepth..MaxDept]
                if (distance < minDepth)
                    distance = minDepth;
                else if (distance > maxDepth)
                    distance = maxDepth;
                *pRangeMap = (uint16_t) ( ( distance - minDepth ) * scale );
            }
            else
            {
                // No depth information available for this pixel. Zero it.
                *pRangeMap = 0;
            }
        }
    }
}

// Calculates a color range map from point cloud data sent from a Basler ToF camera.
// The buffer that pRangeMap points to must be allocated accordingaly before
// passing it to the calculateRangeMap function.
#pragma pack(push, 1)
struct BGR 
{
    uint8_t b;
    uint8_t g;
    uint8_t r;
};
#pragma pack(pop)


void calculateRangeMapColor(const PartInfo& pointCloud, int minDepth, int maxDepth, BGR* pRangeMap)
{
    const int width = (int) pointCloud.width;
    const int height = (int) pointCloud.height;
    CBlazeCamera::Coord3D *pPoint = reinterpret_cast<CBlazeCamera::Coord3D*>(pointCloud.pData);

    const double scale = 65535.0 / (maxDepth - minDepth);

    for (int row = 0; row < height; ++row)
    {
        for (int col = 0; col < width; ++col, ++pPoint, ++pRangeMap)
        {
            if (pPoint->IsValid())
            {
                // calculate the radial distance
                double distance = sqrt(pPoint->x * pPoint->x + pPoint->y * pPoint->y + pPoint->z * pPoint->z);

                // clip to [minDepth..MaxDept]
                if (distance < minDepth)
                    distance = minDepth;
                else if (distance > maxDepth)
                    distance = maxDepth;
                
                // Calculate the color
                BGR bgr;
                const uint16_t g = (uint16_t)((distance - minDepth) * scale);
                const uint16_t val = g >> 6 & 0xff;
                const uint16_t sel = g >> 14;
                uint32_t res = val << 8 | 0xff;
                if (sel & 0x01)
                {
                    res = (~res) >> 8 & 0xffff;
                }
                if (sel & 0x02)
                {
                    res = res << 8;
                }
                bgr.r = res & 0xff;
                res = res >> 8;
                bgr.g = res & 0xff;
                res = res >> 8;
                bgr.b = res & 0xff;

                *pRangeMap = bgr;
            }
            else
            {
                // No depth information available for this pixel. Set it to black.
                BGR bgr;
                bgr.r = bgr.g = bgr.b = 0;
                *pRangeMap = bgr;
            }
        }
    }
}

class Sample
{
public:
    int run();

protected:
    bool onImageGrabbed(GrabResult grabResult, BufferParts);

private:
    CBlazeCamera     m_Camera;   // the ToF camera
    int             m_minDepth; // minimum distance value [mm]
    int             m_maxDepth; // maximum distance value [mm]
};

int Sample::run()
{
    try
    {
        m_Camera.OpenFirstCamera();

        // Query the minimum and maximum distance parameters.
        m_minDepth = (int) GenApi::CIntegerPtr(m_Camera.GetParameter("DepthMin"))->GetValue();
        m_maxDepth = (int) GenApi::CIntegerPtr(m_Camera.GetParameter("DepthMax"))->GetValue();

        // Prepare windows for displaying intensity image and range map.
        namedWindow("Intensity", WINDOW_AUTOSIZE);
        namedWindow("Range Map", WINDOW_AUTOSIZE);
        namedWindow("Range Map Color", WINDOW_AUTOSIZE);

        // Acquire and process images until the call-back onImageGrabbed indicates to stop acquisition. 
        m_Camera.GrabContinuous(2, 1000, this, &Sample::onImageGrabbed);

        // Clean-up
        m_Camera.Close();
    }
    catch (const GenICam::GenericException& e)
    {
        cerr << "Exception occurred: " << e.GetDescription() << endl;
        return EXIT_FAILURE;
    }

    return EXIT_SUCCESS;
}

// Called for each frame grabbed.
bool Sample::onImageGrabbed(GrabResult grabResult, BufferParts parts)
{
    int key = 0;
    if (grabResult.status == GrabResult::Timeout)
    {
        cerr << "Timeout occurred. Acquisition stopped." << endl;
        return false; // Indicate to stop acquisition
    }
    if (grabResult.status != GrabResult::Ok)
    {
        cerr << "Image was not grabbed." << endl;
    }
    else
    {
        try
        {
            // -------  Create OpenCV images from grabbed buffer

            // First the intensity image...
            const int width = (int)parts[0].width;
            const int height = (int)parts[0].height;
            const int count = width * height;
            Mat intensity = Mat(height, width, CV_16UC1, parts[1].pData);
            // Scale the intensity image since it often looks quite dark
            double  max;
            minMaxLoc(intensity, NULL, &max);
            intensity /= (max / std::numeric_limits<uint16_t>::max());

            // ... then the range images.
            uint16_t* pRangeData = new uint16_t[width * height];
            calculateRangeMap(parts[0], m_minDepth, m_maxDepth, pRangeData);
            Mat rangeMap = Mat(height, width, CV_16UC1, pRangeData); 
            
            BGR* pRangeDataColor = new BGR[width * height];
            calculateRangeMapColor(parts[0], m_minDepth, m_maxDepth, pRangeDataColor);
            Mat3b rangeMapColor = Mat(height, width, CV_8UC3, pRangeDataColor);

            // -------   Display the images
            imshow("Intensity", intensity);
            imshow("Range Map", rangeMap);
            imshow("Range Map Color", rangeMapColor);

            // ------ Since rangeMap and rangeMapColor didn't take ownership of the memory, 
            //        we must free the memory to prevent memory leaks.
            delete[] pRangeData;
            delete[] pRangeDataColor;
        }
        catch (const Exception&  e)
        {
            cerr << e.what() << endl;
        }
    }
    key = waitKey(1);
    return 'q' != (char)key;
}

int main(int argc, char* argv[])
{
    int exitCode = EXIT_SUCCESS;
    try
    {
        CBlazeCamera::InitProducer();
        Sample processing;
        exitCode = processing.run();
    }
    catch (GenICam::GenericException& e)
    {
        cerr << "Exception occurred: " << endl << e.GetDescription() << endl;
        exitCode = EXIT_FAILURE;
    }

    // Release the GenTL producer and all of its resources. 
    // Note: Don't call TerminateProducer() until the destructor of the CToFCamera
    // class has been called. The destructor may require resources which may not
    // be available anymore after TerminateProducer() has been called.
    if (CBlazeCamera::IsProducerInitialized())
        CBlazeCamera::TerminateProducer();  // Won't throw any exceptions

    return exitCode;
}
