##############################################################################
# Find Basler API for blaze cameras
#
# This module sets the following variables:
# BaslerBlaze_FOUND
# BaslerBlaze_INCLUDE_DIR
# BaslerBlaze_LIBRARIES
# BaslerBlaze_LINKER_FLAGS
#
# This module exports the following target:
#   BaslerBlaze::BaslerBlazeSDK

# Set BaslerBlaze_ROOT_DIR cmake variable to suitable value.
function(basler_tof_detect_root)
  # Check CMake variable
  string(COMPARE NOTEQUAL "${BaslerBlaze_ROOT_DIR}" "" not_empty)
  if(not_empty)
    set(BaslerBlaze_ROOT_DIR "${BaslerBlaze_ROOT_DIR}" PARENT_SCOPE)
    return()
  endif()

  # Check environment variable
  string(COMPARE NOTEQUAL "$ENV{BASLER_BLAZE_SDKROOT}" "" not_empty)
  if(not_empty)
    set(BaslerBlaze_ROOT_DIR "$ENV{BASLER_BLAZE_SDKROOT}" PARENT_SCOPE)
    return()
  endif()
  if(UNIX)
    set(BaslerBlaze_ROOT_DIR "/opt/Basler_blaze" PARENT_SCOPE)
  endif()
endfunction(basler_tof_detect_root)

# Start detection of BaslerBlaze



basler_tof_detect_root() 
get_filename_component(BaslerBlaze_ROOT_DIR "${BaslerBlaze_ROOT_DIR}" ABSOLUTE) # beautify path
set(BaslerBlaze_ROOT_DIR ${BaslerBlaze_ROOT_DIR} CACHE PATH "Installation location of the Basler blaze SDK.")

if(MSVC) 
    if (CMAKE_CL_64)
        set (BaslerBlaze_LIBDIR ${BaslerBlaze_ROOT_DIR}/lib/x64)
        set (BaslerBlaze_BINDIR ${BaslerBlaze_ROOT_DIR}/bin/x64)
    else()
        set (BaslerBlaze_LIBDIR ${BaslerBlaze_ROOT_DIR}/lib/win32)
        set (BaslerBlaze_BINDIR ${BaslerBlaze_ROOT_DIR}/bin/win32)
    endif()
elseif("${CMAKE_SYSTEM_NAME}" STREQUAL "Linux")
    if( CMAKE_SYSTEM_PROCESSOR MATCHES "arm*" )
        if( CMAKE_SIZEOF_VOID_P EQUAL 4 )
            set(PLATFORM_PATH_NAME Linux32_ARM)
        else( CMAKE_SIZEOF_VOID_P EQUAL 4 )
            set(PLATFORM_PATH_NAME Linux64_ARM)
        endif()
    elseif( CMAKE_SYSTEM_PROCESSOR MATCHES "ppc64")
        set(PLATFORM_PATH_NAME Linux64_PPC64)
    else()
        if( CMAKE_SIZEOF_VOID_P EQUAL 4 )
            set(PLATFORM_PATH_NAME Linux32_i86)
        else( CMAKE_SIZEOF_VOID_P EQUAL 4 )
            set(PLATFORM_PATH_NAME Linux64_x64)
        endif()
    endif()
    if( ARM_FLOATING_POINT MATCHES "hardfloat" )
        set(PLATFORM_PATH_NAME Linux32_ARMhf)
    elseif( ARM_FLOATING_POINT MATCHES "softfloat")
        set(PLATFORM_PATH_NAME Linux32_ARMsf)
    endif()
    set (BaslerBlaze_LIBDIR ${BaslerBlaze_ROOT_DIR}/library/CPP/lib/${PLATFORM_PATH_NAME})
    set (BaslerBlaze_BINDIR ${BaslerBlaze_LIBDIR})
endif()

if(MSVC)
  set (BASLER_GENAPI_VER v3_1_Basler_pylon_v5_1)
  set (GENICAM_COMPILER_INFIX MD_VC141)
else()
  set (BASLER_GENAPI_VER v3_1_Basler_ToF)
  set (GENICAM_COMPILER_INFIX gcc54)
endif()


set (GenApi_Lib_Suffix _${GENICAM_COMPILER_INFIX}_${BASLER_GENAPI_VER} )
set ( BASLER_GENAPI GenApi${GenApi_Lib_Suffix} )
set ( BASLER_GCBASE GCBase${GenApi_Lib_Suffix} )

find_library(BASLER_GENAPI_LIBRARY ${BASLER_GENAPI} PATHS ${BaslerBlaze_LIBDIR} NO_DEFAULT_PATH)
find_library(BASLER_GCBASE_LIBRARY ${BASLER_GCBASE} PATHS ${BaslerBlaze_LIBDIR} NO_DEFAULT_PATH)

if (UNIX)
   find_library(DL dl)
   set(DELAYIMP)
endif()

if (MSVC)
    set(DL)
	set (DELAYIMP delayimp.lib)
endif()

find_path(BASLER_GENAPI_INCLUDE_DIR GenICam.h PATHS ${BaslerBlaze_ROOT_DIR}/include)

include(FindPackageHandleStandardArgs)
find_package_handle_standard_args(BaslerBlaze "Could NOT find the Basler blaze SDK. Set the BaslerBlaze_ROOT_DIR variable \
    to the location you installed the Basler blaze Driver software."
    BASLER_GENAPI_LIBRARY BASLER_GCBASE_LIBRARY BASLER_GENAPI_INCLUDE_DIR)

if (BaslerBlaze_FOUND)
    message(STATUS "Found Basler blaze SDK in " ${BaslerBlaze_ROOT_DIR})
    get_filename_component (BASLER_GENAPI_LIBRARY_DIR ${BASLER_GENAPI_LIBRARY} PATH)
    set (BASLER_GENAPI_LIBRARIES ${BASLER_GENAPI_LIBRARY} ${BASLER_GCBASE_LIBRARY})
    set (BaslerBlaze_LIBRARIES ${BASLER_GENAPI_LIBRARIES} ${DL} ${DELAYIMP} )
    set (BaslerBlaze_INCLUDE_DIRS ${BASLER_GENAPI_INCLUDE_DIR})
    if(UNIX)
        set(BaslerBlaze_DEFINITIONS  )
        set(BaslerBlaze_CXX_FLAGS -std=c++11)
        set(BaslerBlaze_LINKER_FLAGS -Wl,-rpath,${BaslerBlaze_LIBDIR} -Wl,-rpath,${BaslerBlaze_ROOT_DIR}/lib/gentlproducer/gtl -Wl,-E)
    else()
        set(BaslerBlaze_DEFINITIONS  )
        set(BaslerBlaze_CXX_FLAGS )
        set(BaslerBlaze_LINKER_FLAGS /DELAYLOAD:${BASLER_GENAPI}.dll /DELAYLOAD:${BASLER_GCBASE}.dll )
    endif()
endif()
    
#    
#export BaslerBlaze::BaslerBlazeSDK target
#

if (BaslerBlaze_FOUND AND NOT TARGET BaslerBlaze::BaslerBlazeSDK)

    foreach (lib GenApi GCBase)
        add_library(BaslerBlaze::${lib} SHARED IMPORTED GLOBAL)
        set_target_properties(BaslerBlaze::${lib} PROPERTIES
                IMPORTED_LINK_INTERFACE_LANGUAGES CXX
                INTERFACE_INCLUDE_DIRECTORIES ${BaslerBlaze_INCLUDE_DIRS}
        )
        if (WIN32)
            set_target_properties(BaslerBlaze::${lib} PROPERTIES
                IMPORTED_IMPLIB "${BaslerBlaze_LIBDIR}/${lib}${GenApi_Lib_Suffix}.lib"
                IMPORTED_LOCATION "${BaslerBlaze_BINDIR}/${lib}${GenApi_Lib_Suffix}.dll"
            )
        else()
            string(TOUPPER ${lib} LIB)
            set_target_properties(BaslerBlaze::${lib} PROPERTIES
                IMPORTED_LOCATION "${BASLER_${LIB}_LIBRARY}"
            )
        endif()
    endforeach()

    add_library(BaslerBlaze::BaslerBlazeSDK INTERFACE IMPORTED GLOBAL)
    set_target_properties(BaslerBlaze::BaslerBlazeSDK PROPERTIES
        INTERFACE_INCLUDE_DIRECTORIES "${BaslerBlaze_INCLUDE_DIRS}"
        INTERFACE_COMPILE_DEFINITIONS "${BaslerBlaze_DEFINITIONS}"
        INTERFACE_COMPILE_OPTIONS "${BaslerBlaze_CXX_FLAGS}"
    )
    if ( ${CMAKE_VERSION} VERSION_LESS "3.13.0")
        set_target_properties(BaslerBlaze::BaslerBlazeSDK PROPERTIES
            INTERFACE_LINK_LIBRARIES "${BaslerBlaze_LINKER_FLAGS}"
        )
    else()
        set_target_properties(BaslerBlaze::BaslerBlazeSDK PROPERTIES
            INTERFACE_LINK_OPTIONS "${BaslerBlaze_LINKER_FLAGS}"
        )
    endif()

    target_link_libraries(BaslerBlaze::BaslerBlazeSDK INTERFACE BaslerBlaze::GCBase BaslerBlaze::GenApi ${DL} ${DELAYIMP})
endif()




